@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2025 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,9 +20,11 @@ sabnzbd.misc - misc classes
 """
 
 import os
+import platform
 import ssl
 import sys
 import logging
+import functools
 import urllib.request
 import urllib.parse
 import re
@@ -31,12 +33,14 @@ import socket
 import time
 import datetime
 import inspect
+import queue
 import ctypes
 import html
 import ipaddress
 import socks
 from threading import Thread
-from typing import Union, Tuple, Any, AnyStr, Optional, List, Dict
+from collections.abc import Iterable
+from typing import Union, Tuple, Any, AnyStr, Optional, List, Dict, Collection
 
 import sabnzbd
 import sabnzbd.getipaddress
@@ -50,11 +54,13 @@ from sabnzbd.constants import (
 )
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.encoding import ubtou
-from sabnzbd.filesystem import userxbit, make_script_path, remove_file, is_valid_script
+from sabnzbd.decorators import cache_maintainer
+from sabnzbd.encoding import ubtou, platform_btou
+from sabnzbd.filesystem import userxbit, make_script_path, remove_file
 
-if sabnzbd.WIN32:
+if sabnzbd.WINDOWS:
     try:
+        import winreg
         import win32process
         import win32con
 
@@ -82,9 +88,17 @@ RE_IP6 = re.compile(r"inet6\s+(addr:\s*)?([0-9a-f:]+)", re.I)
 HAVE_AMPM = bool(time.strftime("%p"))
 
 
-def helpful_warning(*args, **kwargs):
+def helpful_warning(msg, *args, **kwargs):
     """Wrapper to ignore helpful warnings if desired"""
-    if sabnzbd.cfg.helpful_warnings():
+    if cfg.helpful_warnings():
+        msg = "%s\n%s" % (msg, T("To prevent all helpful warnings, disable Special setting 'helpful_warnings'."))
+        return logging.warning(msg, *args, **kwargs)
+    return logging.info(msg, *args, **kwargs)
+
+
+def duplicate_warning(*args, **kwargs):
+    """Wrapper to ignore duplicate warnings if desired"""
+    if cfg.warn_dupl_jobs():
         return logging.warning(*args, **kwargs)
     return logging.info(*args, **kwargs)
 
@@ -137,33 +151,42 @@ def calc_age(date: datetime.datetime, trans: bool = False) -> str:
         d = "d"
         h = "h"
         m = "m"
+
     try:
-        now = datetime.datetime.now()
-        # age = str(now - date).split(".")[0] #old calc_age
-
-        # time difference
-        dage = now - date
-        seconds = dage.seconds
-        # only one value should be returned
-        # if it is less than 1 day then it returns in hours, unless it is less than one hour where it returns in minutes
-        if dage.days:
-            age = "%d%s" % (dage.days, d)
-        elif int(seconds / 3600):
-            age = "%d%s" % (seconds / 3600, h)
+        # Return time difference in human-readable format
+        date_diff = datetime.datetime.now() - date
+        if date_diff.days:
+            return "%d%s" % (date_diff.days, d)
+        elif int(date_diff.seconds / 3600):
+            return "%d%s" % (date_diff.seconds / 3600, h)
         else:
-            age = "%d%s" % (seconds / 60, m)
+            return "%d%s" % (date_diff.seconds / 60, m)
     except:
-        age = "-"
-
-    return age
+        return "-"
 
 
 def safe_lower(txt: Any) -> str:
     """Return lowercased string. Return '' for None"""
-    if txt:
+    if txt := str_conv(txt):
         return txt.lower()
-    else:
-        return ""
+    return ""
+
+
+def is_none(inp: Any) -> bool:
+    """Check for 'not X' but also if it's maybe the string 'None'"""
+    return not inp or (isinstance(inp, str) and inp.lower() == "none")
+
+
+def clean_comma_separated_list(inp: Any) -> List[str]:
+    """Return a list of stripped values from a string or list, empty ones removed"""
+    result_ids = []
+    if isinstance(inp, str):
+        inp = inp.split(",")
+    if isinstance(inp, Iterable):
+        for inp_id in inp:
+            if new_id := inp_id.strip():
+                result_ids.append(new_id)
+    return result_ids
 
 
 def cmp(x, y):
@@ -178,6 +201,38 @@ def cmp(x, y):
     return (x > y) - (x < y)
 
 
+class MultiAddQueue(queue.Queue):
+    def put_multiple(self, multiple_items: Collection):
+        """Take advantage of the dequeue used by Queue that has a very
+        fast extend method to add multiple items at once.
+        See: https://github.com/sabnzbd/sabnzbd/discussions/2704"""
+        with self.not_full:
+            self.queue.extend(multiple_items)
+            self.unfinished_tasks += len(multiple_items)
+            self.not_empty.notify()
+
+
+def cat_pp_script_sanitizer(
+    cat: Optional[str] = None,
+    pp: Optional[Union[int, str]] = None,
+    script: Optional[str] = None,
+) -> Tuple[Optional[Union[int, str]], Optional[str], Optional[str]]:
+    """Basic sanitizer from outside input to a bit more predictable values"""
+    # * and Default are valid values
+    if safe_lower(cat) in ("", "none"):
+        cat = None
+
+    # Cannot use "not pp" because pp can also be 0
+    if safe_lower(pp) in ("", "-1", "none"):
+        pp = None
+
+    # Check for valid script is performed in NzbObject init
+    if not script or safe_lower(script) == "default":
+        script = None
+
+    return cat, pp, script
+
+
 def name_to_cat(fname, cat=None):
     """Retrieve category from file name, but only if "cat" is None."""
     if cat is None and fname.startswith("{{"):
@@ -223,10 +278,10 @@ def cat_to_opts(cat, pp=None, script=None, priority=None) -> Tuple[str, int, str
     return cat, pp, script, priority
 
 
-def pp_to_opts(pp: int) -> Tuple[bool, bool, bool]:
+def pp_to_opts(pp: Optional[int]) -> Tuple[bool, bool, bool]:
     """Convert numeric processing options to (repair, unpack, delete)"""
     # Convert the pp to an int
-    pp = sabnzbd.interface.int_conv(pp)
+    pp = int_conv(pp)
     if pp == 0:
         return False, False, False
     if pp == 1:
@@ -303,7 +358,7 @@ def cat_convert(cat):
     If no match found, but the indexer-cat starts with the user-cat, return user-cat
     If no match found, return None
     """
-    if cat and cat.lower() != "none":
+    if not is_none(cat):
         cats = config.get_ordered_categories()
         raw_cats = config.get_categories()
         for ucat in cats:
@@ -343,8 +398,6 @@ _SERVICE_PARM = "CommandLine"
 
 def get_serv_parms(service):
     """Get the service command line parameters from Registry"""
-    import winreg
-
     service_parms = []
     try:
         key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, _SERVICE_KEY + service)
@@ -364,8 +417,6 @@ def get_serv_parms(service):
 
 def set_serv_parms(service, args):
     """Set the service command line parameters in Registry"""
-    import winreg
-
     serv = []
     for arg in args:
         serv.append(arg[0])
@@ -396,8 +447,7 @@ def convert_version(text):
     """Convert version string to numerical value and a testversion indicator"""
     version = 0
     test = True
-    m = RE_VERSION.search(ubtou(text))
-    if m:
+    if m := RE_VERSION.search(ubtou(text)):
         version = int(m.group(1)) * 1000000 + int(m.group(2)) * 10000 + int(m.group(3)) * 100
         try:
             if m.group(4).lower() == "rc":
@@ -445,7 +495,7 @@ def check_latest_version():
     # Fetch version info
     data = get_from_url("https://sabnzbd.org/latest.txt")
     if not data:
-        logging.info("Cannot retrieve version information from GitHub.com")
+        logging.info("Cannot retrieve version information from sabnzbd.org")
         logging.debug("Traceback: ", exc_info=True)
         return
 
@@ -525,8 +575,8 @@ def from_units(val: str) -> float:
     val = str(val).strip().upper()
     if val == "-1":
         return float(val)
-    m = RE_UNITS.search(val)
-    if m:
+
+    if m := RE_UNITS.search(val):
         if m.group(2):
             val = float(m.group(1))
             unit = m.group(2)
@@ -639,7 +689,7 @@ def get_cache_limit():
     """
     # Calculate, if possible
     try:
-        if sabnzbd.WIN32:
+        if sabnzbd.WINDOWS:
             # Windows
             mem_bytes = get_windows_memory()
         elif sabnzbd.MACOS:
@@ -663,7 +713,7 @@ def get_cache_limit():
         pass
 
     # Always at least minimum on Windows/macOS
-    if sabnzbd.WIN32 and sabnzbd.MACOS:
+    if sabnzbd.WINDOWS and sabnzbd.MACOS:
         return DEF_ARTICLE_CACHE_DEFAULT
 
     # If failed, leave empty for Linux so user needs to decide
@@ -702,6 +752,94 @@ def get_macos_memory():
     return float(system_output.split()[1])
 
 
+@cache_maintainer(clear_time=3600)
+@functools.lru_cache(maxsize=None)
+def get_cpu_name():
+    """Find the CPU name (which needs a different method per OS), and return it
+    If none found, return platform.platform()"""
+
+    cputype = None
+
+    try:
+        if sabnzbd.WINDOWS:
+            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"Hardware\Description\System\CentralProcessor\0")
+            cputype = winreg.QueryValueEx(key, "ProcessorNameString")[0]
+            winreg.CloseKey(key)
+
+        elif sabnzbd.MACOS:
+            cputype = subprocess.check_output(["sysctl", "-n", "machdep.cpu.brand_string"]).strip()
+
+        else:
+            with open("/proc/cpuinfo") as fp:
+                for myline in fp.readlines():
+                    if myline.startswith("model name"):
+                        # Typical line:
+                        # model name      : Intel(R) Xeon(R) CPU           E5335  @ 2.00GHz
+                        cputype = myline.split(":", 1)[1]  # get everything after the first ":"
+                        break  # we're done
+        cputype = platform_btou(cputype)
+    except:
+        # An exception, maybe due to a subprocess call gone wrong
+        pass
+
+    if cputype:
+        # OK, found. Remove unwanted spaces:
+        cputype = " ".join(cputype.split())
+    else:
+        try:
+            # Not found, so let's fall back to platform()
+            cputype = platform.platform()
+        except:
+            # Can fail on special platforms (like Snapcraft or embedded)
+            pass
+
+    logging.debug("CPU model = %s", cputype)
+    return cputype
+
+
+def get_platform_description() -> str:
+    """Get a nicer description of what platform we are on"""
+    platform_tags = []
+    # We dig deeper for Linux
+    if not sabnzbd.WINDOWS and not sabnzbd.MACOS:
+        # Check if running in a Docker container
+        # Note: fake-able, but good enough for normal setups
+        if os.path.exists("/.dockerenv"):
+            platform_tags.append("Docker")
+            # See if we are on Unraid
+            if "HOST_OS" in os.environ and os.environ["HOST_OS"].lower() == "unraid":
+                platform_tags.append("Unraid")
+        elif "container" in os.environ:
+            platform_tags.append("Flatpak")
+        elif "APPIMAGE" in os.environ:
+            platform_tags.append("AppImage")
+        elif "SNAP" in os.environ:
+            platform_tags.append("Snap")
+        else:
+            # Check for other forms of virtualization
+            try:
+                if virt := run_command(["systemd-detect-virt"], stderr=subprocess.DEVNULL).strip():
+                    if virt != "none":
+                        platform_tags.append(virt)
+            except:
+                pass
+
+            try:
+                # Only present in Python 3.10+
+                # Can print nicer description like "Ubuntu 24.02 LTS"
+                platform_tags.append(platform.freedesktop_os_release()["PRETTY_NAME"])
+            except:
+                pass
+
+    if not platform_tags:
+        # Fallback if we found nothing or on Windows/macOS
+        platform_tags.append(platform.platform(terse=True))
+
+    # Add all together
+    sabnzbd.PLATFORM = " ".join(platform_tags)
+    return sabnzbd.PLATFORM
+
+
 def on_cleanup_list(filename: str, skip_nzb: bool = False) -> bool:
     """Return True if a filename matches the clean-up list"""
     cleanup_list = cfg.cleanup_list()
@@ -741,7 +879,7 @@ _HAVE_STATM = _PAGE_SIZE and memory_usage()
 def loadavg():
     """Return 1, 5 and 15 minute load average of host or "" if not supported"""
     p = ""
-    if not sabnzbd.WIN32 and not sabnzbd.MACOS:
+    if not sabnzbd.WINDOWS and not sabnzbd.MACOS:
         try:
             p = "%.2f | %.2f | %.2f" % os.getloadavg()
         except:
@@ -787,13 +925,30 @@ def format_time_string(seconds: float) -> str:
     return " ".join(completestr)
 
 
-def int_conv(value: Any) -> int:
-    """Safe conversion to int (can handle None)"""
+def str_conv(value: Any, default: str = "") -> str:
+    """Safe conversion to str (None will be converted to empty string)
+    Returns empty string or requested default value"""
+    if value is None:
+        return default
     try:
-        value = int(value)
+        return str(value)
     except:
-        value = 0
-    return value
+        return default
+
+
+def int_conv(value: Any, default: int = 0) -> int:
+    """Safe conversion to int (can handle None)
+    Returns 0 or requested default value"""
+    try:
+        return int(value)
+    except:
+        return default
+
+
+def bool_conv(value: Any) -> bool:
+    """Safe conversion to bool (can handle None)
+    Returns False in case of None or non-convertable value"""
+    return bool(int_conv(value))
 
 
 def create_https_certificates(ssl_cert, ssl_key):
@@ -877,7 +1032,7 @@ def is_sample(filename: str) -> bool:
 
 def find_on_path(targets):
     """Search the PATH for a program and return full path"""
-    if sabnzbd.WIN32:
+    if sabnzbd.WINDOWS:
         paths = os.getenv("PATH").split(";")
     else:
         paths = os.getenv("PATH").split(":")
@@ -985,6 +1140,15 @@ def is_lan_addr(ip: str) -> bool:
         return False
 
 
+def is_local_addr(ip: str) -> bool:
+    """Determine if an IP address is to be considered local, i.e. it's part of a subnet in
+    local_ranges, if defined, or in private address space reserved for local area networks."""
+    if local_ranges := cfg.local_ranges():
+        return any(ip_in_subnet(ip, local_range) for local_range in local_ranges)
+    else:
+        return is_lan_addr(ip)
+
+
 def ip_extract() -> List[str]:
     """Return list of IP addresses of this system"""
     ips = []
@@ -996,7 +1160,7 @@ def ip_extract() -> List[str]:
         if program:
             program = [program]
 
-    if sabnzbd.WIN32 or not program:
+    if sabnzbd.WINDOWS or not program:
         try:
             info = socket.getaddrinfo(socket.gethostname(), None)
         except:
@@ -1015,42 +1179,6 @@ def ip_extract() -> List[str]:
     return ips
 
 
-def get_server_addrinfo(host: str, port: int) -> socket.getaddrinfo:
-    """Return processed getaddrinfo()"""
-    try:
-        int(port)
-    except:
-        port = 119
-    opt = sabnzbd.cfg.ipv6_servers()
-    """ ... with the following meaning for 'opt':
-    Control the use of IPv6 Usenet server addresses. Meaning:
-    0 = don't use
-    1 = use when available and reachable (DEFAULT)
-    2 = force usage (when SABnzbd's detection fails)
-    """
-    try:
-        # Standard IPV4 or IPV6
-        ips = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
-        if opt == 2 or (opt == 1 and sabnzbd.EXTERNAL_IPV6) or (opt == 1 and sabnzbd.cfg.load_balancing() == 2):
-            # IPv6 forced by user, or IPv6 allowed and reachable, or IPv6 allowed and loadbalancing-with-IPv6 activated
-            # So return all IP addresses, no matter IPv4 or IPv6:
-            return ips
-        else:
-            # IPv6 unreachable or not allowed by user, so only return IPv4 address(es):
-            return [ip for ip in ips if ":" not in ip[4][0]]
-    except:
-        if opt == 2 or (opt == 1 and sabnzbd.EXTERNAL_IPV6) or (opt == 1 and sabnzbd.cfg.load_balancing() == 2):
-            try:
-                # Try IPV6 explicitly
-                return socket.getaddrinfo(
-                    host, port, socket.AF_INET6, socket.SOCK_STREAM, socket.IPPROTO_IP, socket.AI_CANONNAME
-                )
-            except:
-                # Nothing found!
-                pass
-        return []
-
-
 def get_base_url(url: str) -> str:
     """Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
     But also api.althub.co.za -> althub.co.za
@@ -1121,7 +1249,7 @@ def build_and_run_command(command: List[str], windows_unrar_command: bool = Fals
         logging.error(T("[%s] The command in build_command is undefined."), caller_name())
         raise IOError
 
-    if not sabnzbd.WIN32:
+    if not sabnzbd.WINDOWS:
         if command[0].endswith(".py"):
             with open(command[0], "r") as script_file:
                 if not userxbit(command[0]):
@@ -1185,15 +1313,14 @@ def run_command(cmd: List[str], **kwargs):
     return txt
 
 
-def run_script(script):
-    """Run a user script (queue complete only)"""
-    script_path = make_script_path(script)
-    if script_path:
+def run_script(script: str):
+    """Run a user script"""
+    if script_path := make_script_path(script):
         try:
-            script_output = run_command([script_path])
-            logging.info("Output of queue-complete script %s: \n%s", script, script_output)
+            script_output = run_command([script_path], env=sabnzbd.newsunpack.create_env())
+            logging.info("Output of script %s: \n%s", script, script_output)
         except:
-            logging.info("Failed queue-complete script %s, Traceback: ", script, exc_info=True)
+            logging.info("Failed script %s, Traceback: ", script, exc_info=True)
 
 
 def set_socks5_proxy():
@@ -1223,61 +1350,6 @@ def set_https_verification(value):
     return prev
 
 
-def test_ipv6():
-    """Check if external IPv6 addresses are reachable"""
-    if not cfg.selftest_host():
-        # User disabled the test, assume active IPv6
-        return True
-    try:
-        info = sabnzbd.getipaddress.addresslookup6(cfg.selftest_host())
-    except:
-        logging.debug(
-            "Test IPv6: Disabling IPv6, because it looks like it's not available. Reason: %s", sys.exc_info()[0]
-        )
-        return False
-
-    try:
-        af, socktype, proto, canonname, sa = info[0]
-        with socket.socket(af, socktype, proto) as sock:
-            sock.settimeout(2)  # 2 second timeout
-            sock.connect(sa[0:2])
-        logging.debug("Test IPv6: IPv6 test successful. Enabling IPv6")
-        return True
-    except socket.error:
-        logging.debug("Test IPv6: Cannot reach IPv6 test host. Disabling IPv6")
-        return False
-    except:
-        logging.debug("Test IPv6: Problem during IPv6 connect. Disabling IPv6. Reason: %s", sys.exc_info()[0])
-        return False
-
-
-def test_cert_checking():
-    """Test quality of certificate validation"""
-    # User disabled the test, assume proper SSL certificates
-    if not cfg.selftest_host():
-        return True
-
-    # Try a connection to our test-host
-    try:
-        ctx = ssl.create_default_context()
-        base_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        ssl_sock = ctx.wrap_socket(base_sock, server_hostname=cfg.selftest_host())
-        ssl_sock.settimeout(2.0)
-        ssl_sock.connect((cfg.selftest_host(), 443))
-        ssl_sock.close()
-        return True
-    except (socket.gaierror, socket.timeout):
-        # Non-SSL related error.
-        # We now assume that certificates work instead of forcing
-        # lower quality just because some (temporary) internet problem
-        logging.info("Could not determine system certificate validation quality due to connection problems")
-        return True
-    except:
-        # Seems something is still wrong
-        set_https_verification(False)
-    return False
-
-
 def request_repair():
     """Request a full repair on next restart"""
     path = os.path.join(cfg.admin_dir.get_path(), REPAIR_REQUEST)
@@ -1309,7 +1381,7 @@ def system_shutdown():
     while sabnzbd.__INITIALIZED__:
         time.sleep(1.0)
 
-    if sabnzbd.WIN32:
+    if sabnzbd.WINDOWS:
         sabnzbd.powersup.win_shutdown()
     elif sabnzbd.MACOS:
         sabnzbd.powersup.osx_shutdown()
@@ -1320,7 +1392,7 @@ def system_shutdown():
 def system_hibernate():
     """Hibernate system"""
     logging.info("Performing system hybernation")
-    if sabnzbd.WIN32:
+    if sabnzbd.WINDOWS:
         sabnzbd.powersup.win_hibernate()
     elif sabnzbd.MACOS:
         sabnzbd.powersup.osx_hibernate()
@@ -1331,7 +1403,7 @@ def system_hibernate():
 def system_standby():
     """Standby system"""
     logging.info("Performing system standby")
-    if sabnzbd.WIN32:
+    if sabnzbd.WINDOWS:
         sabnzbd.powersup.win_standby()
     elif sabnzbd.MACOS:
         sabnzbd.powersup.osx_standby()
@@ -1340,24 +1412,17 @@ def system_standby():
 
 
 def change_queue_complete_action(action: str, new: bool = True):
-    """Action or script to be performed once the queue has been completed
-    Scripts are prefixed with 'script_'
-    """
-    _action = None
-    _argument = None
+    """Action or script to be performed once the queue has been completed"""
+    function = None
     if new or cfg.queue_complete_pers():
-        if action.startswith("script_") and is_valid_script(action.replace("script_", "", 1)):
-            # all scripts are labeled script_xxx
-            _action = sabnzbd.misc.run_script
-            _argument = action.replace("script_", "", 1)
-        elif action == "shutdown_pc":
-            _action = system_shutdown
+        if action == "shutdown_pc":
+            function = system_shutdown
         elif action == "hibernate_pc":
-            _action = system_hibernate
+            function = system_hibernate
         elif action == "standby_pc":
-            _action = system_standby
+            function = system_standby
         elif action == "shutdown_program":
-            _action = sabnzbd.shutdown_program
+            function = sabnzbd.shutdown_program
         else:
             action = None
     else:
@@ -1368,8 +1433,7 @@ def change_queue_complete_action(action: str, new: bool = True):
         config.save_config()
 
     sabnzbd.QUEUECOMPLETE = action
-    sabnzbd.QUEUECOMPLETEACTION = _action
-    sabnzbd.QUEUECOMPLETEARG = _argument
+    sabnzbd.QUEUECOMPLETEACTION = function
 
 
 def keep_awake():
@@ -1485,3 +1549,18 @@ def convert_sorter_settings():
         # Configure the new sorter
         logging.debug("Converted old movie sorter config to '%s': %s", T("Movie Sorting"), movie_sorter)
         config.ConfigSorter(T("Movie Sorting"), movie_sorter)
+
+
+def convert_history_retention():
+    """Convert single-option to the split history retention setting"""
+    if "d" in cfg.history_retention():
+        days_to_keep = int_conv(cfg.history_retention().strip()[:-1])
+        cfg.history_retention_option.set("days-delete")
+        cfg.history_retention_number.set(days_to_keep)
+    else:
+        to_keep = int_conv(sabnzbd.cfg.history_retention())
+        if to_keep > 0:
+            cfg.history_retention_option.set("number-delete")
+            cfg.history_retention_number.set(to_keep)
+        elif to_keep < 0:
+            cfg.history_retention_option.set("all-delete")
