@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2024 by The SABnzbd-Team (sabnzbd.org)
+# Copyright 2007-2025 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -70,7 +70,6 @@ from sabnzbd.filesystem import (
     set_permissions,
     long_path,
     fix_unix_encoding,
-    get_ext,
     get_filename,
     get_unique_filename,
     renamer,
@@ -89,8 +88,9 @@ from sabnzbd.filesystem import (
     backup_nzb,
     remove_data,
     strip_extensions,
+    get_ext,
 )
-from sabnzbd.par2file import FilePar2Info
+from sabnzbd.par2file import FilePar2Info, has_par2_in_filename, analyse_par2, parse_par2_file, is_par2_file
 from sabnzbd.decorators import synchronized
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
@@ -111,7 +111,7 @@ RE_RAR = re.compile(r"(\.rar|\.r\d\d|\.s\d\d|\.t\d\d|\.u\d\d|\.v\d\d)$", re.I)
 # Trylist
 ##############################################################################
 
-TRYLIST_LOCK = threading.Lock()
+TRYLIST_LOCK = threading.RLock()
 
 
 class TryList:
@@ -203,12 +203,25 @@ class Article(TryList):
         self.crc32: Optional[int] = None
         self.nzf: NzbFile = nzf
 
+    @synchronized(TRYLIST_LOCK)
     def reset_try_list(self):
-        """In addition to resetting the try list, also reset fetcher so all servers are tried again"""
+        """In addition to resetting the try list, also reset fetcher so all servers
+        are tried again. Locked so fetcher setting changes are also protected."""
         self.fetcher = None
         self.fetcher_priority = 0
         super().reset_try_list()
 
+    @synchronized(TRYLIST_LOCK)
+    def allow_new_fetcher(self, remove_fetcher_from_try_list: bool = True):
+        """Let article get new fetcher and reset try lists of file and job.
+        Locked so all resets are performed at once"""
+        if remove_fetcher_from_try_list:
+            self.remove_from_try_list(self.fetcher)
+        self.fetcher = None
+        self.tries = 0
+        self.nzf.reset_try_list()
+        self.nzf.nzo.reset_try_list()
+
     def get_article(self, server: Server, servers: List[Server]):
         """Return article when appropriate for specified server"""
         if self.fetcher or self.server_in_try_list(server):
@@ -249,7 +262,7 @@ class Article(TryList):
                 if server.priority >= self.fetcher.priority:
                     self.tries = 0
                     # Allow all servers for this nzo and nzf again (but not this fetcher for this article)
-                    sabnzbd.NzbQueue.reset_try_lists(self, remove_fetcher_from_trylist=False)
+                    self.allow_new_fetcher(remove_fetcher_from_try_list=False)
                     return True
 
         logging.info("Article %s unavailable on all servers, discarding", self.article)
@@ -276,17 +289,6 @@ class Article(TryList):
         self.fetcher_priority = 0
         self.tries = 0
 
-    def __eq__(self, other):
-        """Articles with the same usenet address are the same"""
-        return self.article == other.article
-
-    def __hash__(self):
-        """Required because we implement eq. Articles with the same
-        usenet address can appear in different NZF's. So we make every
-        article object unique, even though it is bad practice.
-        """
-        return id(self)
-
     def __repr__(self):
         return "<Article: article=%s, bytes=%s, art_id=%s>" % (self.article, self.bytes, self.art_id)
 
@@ -294,6 +296,10 @@ class Article(TryList):
 ##############################################################################
 # NzbFile
 ##############################################################################
+class SkippedNzbFile(Exception):
+    pass
+
+
 NzbFileSaver = (
     "date",
     "filename",
@@ -311,7 +317,6 @@ NzbFileSaver = (
     "nzo",
     "nzf_id",
     "deleted",
-    "valid",
     "import_finished",
     "crc32",
     "assembled",
@@ -323,7 +328,7 @@ class NzbFile(TryList):
     """Representation of one file consisting of multiple articles"""
 
     # Pre-define attributes to save memory
-    __slots__ = NzbFileSaver + ("first_article",)
+    __slots__ = NzbFileSaver
 
     def __init__(self, date, subject, raw_article_db, file_bytes, nzo):
         """Setup object"""
@@ -349,7 +354,6 @@ class NzbFile(TryList):
         self.bytes_left: int = file_bytes
 
         self.nzo: NzbObject = nzo
-        self.nzf_id: str = get_new_id("nzf", nzo.admin_path)
         self.deleted = False
         self.import_finished = False
 
@@ -357,26 +361,29 @@ class NzbFile(TryList):
         self.assembled: bool = False
         self.md5of16k: Optional[bytes] = None
 
-        self.valid: bool = bool(raw_article_db)
+        # Add first article to decodetable, this way we can check
+        # if this is maybe a duplicate nzf
+        if raw_article_db:
+            first_article = self.add_article(raw_article_db.pop(0))
+            first_article.lowest_partnum = True
 
-        # Temporarily hold the first article during import
-        self.first_article: Optional[Article] = None
+        if self in nzo.files:
+            logging.info("File %s occurred twice in NZB, skipping", self.filename)
+            raise SkippedNzbFile
 
-        if self.valid and self.nzf_id:
-            # Save first article separate, so we can deobfuscate-during-download
-            # We process the first_file in nzo.add_nzf because if this NZF turns
-            # out to be a duplicate file inside the NZB, the first article would
-            # otherwise become a ghost article.
-            self.first_article = self.add_article(raw_article_db.pop(0))
-            self.first_article.lowest_partnum = True
+        # Create file on disk, which can fail in case of disk errors
+        self.nzf_id: str = get_new_id("nzf", nzo.admin_path)
+        if not self.nzf_id:
+            # Error already shown to user from get_new_id
+            raise SkippedNzbFile
 
-            # Any articles left?
-            if raw_article_db:
-                # Save the rest
-                save_data(raw_article_db, self.nzf_id, nzo.admin_path)
-            else:
-                # All imported
-                self.import_finished = True
+        # Any articles left?
+        if raw_article_db:
+            # Save the rest
+            save_data(raw_article_db, self.nzf_id, nzo.admin_path)
+        else:
+            # All imported
+            self.import_finished = True
 
     def finish_import(self):
         """Load the article objects from disk"""
@@ -408,7 +415,7 @@ class NzbFile(TryList):
         return len(self.articles)
 
     def set_par2(self, setname, vol, blocks):
-        """Designate this this file as a par2 file"""
+        """Designate this file as a par2 file"""
         self.is_par2 = True
         self.setname = setname
         self.vol = vol
@@ -431,8 +438,11 @@ class NzbFile(TryList):
         self.add_to_try_list(server)
         return articles
 
+    @synchronized(TRYLIST_LOCK)
     def reset_all_try_lists(self):
-        """Clear all lists of visited servers"""
+        """Reset all try lists. Locked so reset is performed
+        for all items at the same time without chance of another
+        thread changing any of the items while we are resetting"""
         for art in self.articles:
             art.reset_try_list()
         self.reset_try_list()
@@ -481,10 +491,12 @@ class NzbFile(TryList):
         """Assume it's the same file if the number bytes and first article
         are the same or if there are no articles left, use the filenames.
         Some NZB's are just a mess and report different sizes for the same article.
+        We used to compare (__eq__) articles based on article-ID, however, this failed
+        because some NZB's had the same article-ID twice within one NZF.
         """
         if other and (self.bytes == other.bytes or len(self.decodetable) == len(other.decodetable)):
             if self.decodetable and other.decodetable:
-                return self.decodetable[0] == other.decodetable[0]
+                return self.decodetable[0].article == other.decodetable[0].article
             # Fallback to filename comparison
             return self.filename == other.filename
         return False
@@ -948,12 +960,13 @@ class NzbObject(TryList):
         self.bytes += nzf.bytes
 
         # Only now add first article to the list
-        self.first_articles.append(nzf.first_article)
-        self.first_articles_count += 1
-        nzf.first_article = None
+        if len(nzf.decodetable):
+            self.first_articles.append(nzf.decodetable[0])
+            self.first_articles_count += 1
 
         # Count how many bytes are available for repair
-        if sabnzbd.par2file.is_parfile(nzf.filename):
+        # This is quite unreliable in case of obfuscation and broken par2 files
+        if has_par2_in_filename(nzf.filename):
             self.bytes_par2 += nzf.bytes
 
         logging.info("File %s added to queue", nzf.filename)
@@ -1001,7 +1014,11 @@ class NzbObject(TryList):
                 except:
                     logging.debug("The lastrar swap did not go well")
 
+    @synchronized(TRYLIST_LOCK)
     def reset_all_try_lists(self):
+        """Reset all try lists. Locked so reset is performed
+        for all items at the same time without chance of another
+        thread changing any of the items while we are resetting"""
         for nzf in self.files:
             nzf.reset_all_try_lists()
         self.reset_try_list()
@@ -1014,22 +1031,21 @@ class NzbObject(TryList):
         if parset not in self.extrapars:
             self.extrapars[parset] = []
 
-        lparset = parset.lower()
         for xnzf in self.files[:]:
-            # Move only when not current NZF and filename was extractable from subject
-            if xnzf.filename:
-                setname, vol, block = sabnzbd.par2file.analyse_par2(xnzf.filename)
-                # Don't postpone header-only-files, so we can extract all
-                # possible md5of16k and par2packs's even if the filenames are bad
-                # Usually they are all downloaded as first_articles
-                if setname and block and matcher(lparset, setname.lower()):
-                    xnzf.set_par2(parset, vol, block)
-                    # Don't postpone if all par2 are desired and should be kept or not repairing
-                    if self.repair and not (cfg.enable_all_par() and not cfg.enable_par_cleanup()):
-                        self.extrapars[parset].append(xnzf)
-                        self.files.remove(xnzf)
-                        # Already count these bytes as done
-                        self.bytes_tried += xnzf.bytes_left
+            # Allow re-analysis in case the name was changed by yEnc-header
+            setname, vol, block = analyse_par2(xnzf.filename)
+            # Don't postpone header-only-files, so we can extract all
+            # possible md5of16k and par2packs's even if the filenames are bad
+            # Usually they are all downloaded as first_articles
+            # We match on exact match
+            if setname and block and parset.lower() == setname.lower():
+                xnzf.set_par2(parset, vol, block)
+                # Don't postpone if all par2 are desired and should be kept or not repairing
+                if self.repair and not (cfg.enable_all_par() and not cfg.enable_par_cleanup()):
+                    self.extrapars[parset].append(xnzf)
+                    self.files.remove(xnzf)
+                    # Already count these bytes as done
+                    self.bytes_tried += xnzf.bytes_left
 
         # Sort the sets
         for setname in self.extrapars:
@@ -1045,11 +1061,11 @@ class NzbObject(TryList):
         self.remove_extrapar(nzf)
 
         # Reparse
-        setname, vol, block = sabnzbd.par2file.analyse_par2(nzf.filename, filepath)
+        setname, vol, block = analyse_par2(nzf.filename, filepath)
         nzf.set_par2(setname, vol, block)
 
         # Parse the file contents for hashes
-        set_id, pack = sabnzbd.par2file.parse_par2_file(filepath, nzf.nzo.md5of16k)
+        set_id, pack = parse_par2_file(filepath, nzf.nzo.md5of16k)
 
         # If we couldn't parse it, we ignore it
         if set_id and pack:
@@ -1095,7 +1111,7 @@ class NzbObject(TryList):
         """In case of a broken par2 or missing par2, move another
         of the same set to the top (if we can find it)
         """
-        setname, vol, block = sabnzbd.par2file.analyse_par2(nzf.filename)
+        setname, vol, block = analyse_par2(nzf.filename)
         # Now we need to identify if we have more in this set
         if setname and self.repair:
             # Maybe it was the first one
@@ -1103,10 +1119,8 @@ class NzbObject(TryList):
                 self.postpone_pars(setname)
             # Get the next one
             for new_nzf in self.extrapars[setname]:
+                # Add it to the top
                 if self.add_parfile(new_nzf):
-                    # Add it to the top
-                    self.files.remove(new_nzf)
-                    self.files.insert(0, new_nzf)
                     break
 
     def get_extra_blocks(self, setname: str, needed_blocks: int) -> int:
@@ -1167,6 +1181,11 @@ class NzbObject(TryList):
             # Increase missing bytes counter
             self.bytes_missing += article.bytes
 
+            # Reduce available par2 data if it's a par2 file
+            # This will be unreliable in case of obfuscated par2 files!
+            if nzf.is_par2:
+                self.bytes_par2 -= article.bytes
+
             # Add extra parfiles when there was a damaged article and not pre-checking
             if self.extrapars and not self.precheck:
                 self.prospective_add(nzf)
@@ -1212,7 +1231,7 @@ class NzbObject(TryList):
         # Abort the job due to failure
         if not job_can_succeed:
             self.fail_msg = T("Aborted, cannot be completed") + " - https://sabnzbd.org/not-complete"
-            self.set_unpack_info("Download", self.fail_msg, unique=False)
+            self.set_unpack_info("Download", self.fail_msg)
             logging.debug('Abort job "%s", due to impossibility to complete it', self.final_name)
             return True, True, True
 
@@ -1286,7 +1305,7 @@ class NzbObject(TryList):
         # otherwise updates the byte-counters incorrectly.
         for nzf in self.finished_files:
             filepath = os.path.join(wdir, nzf.filename)
-            if sabnzbd.par2file.is_parfile(filepath):
+            if is_par2_file(filepath):
                 self.handle_par2(nzf, filepath)
                 self.bytes_par2 += nzf.bytes
 
@@ -1442,13 +1461,14 @@ class NzbObject(TryList):
     @synchronized(NZO_LOCK)
     def add_parfile(self, parfile: NzbFile) -> bool:
         """Add parfile to the files to be downloaded
-        Resets trylist just to be sure
+        Add it to the start so we try it first
+        Resets try list just to be sure
         Adjust download-size accordingly
         Returns False when the file couldn't be added
         """
         if not parfile.completed and parfile not in self.files and parfile not in self.finished_files:
             parfile.reset_try_list()
-            self.files.append(parfile)
+            self.files.insert(0, parfile)
             self.bytes_tried -= parfile.bytes_left
             return True
         return False
@@ -1750,7 +1770,7 @@ class NzbObject(TryList):
             yenc_filename
             and yenc_filename != nzf.filename
             and not is_probably_obfuscated(yenc_filename)
-            and not nzf.filename.endswith(".par2")
+            and not sabnzbd.par2file.has_par2_in_filename(nzf.filename)
         ):
             logging.info("Detected filename from yenc or uu: %s -> %s", nzf.filename, yenc_filename)
             self.renamed_file(yenc_filename, nzf.filename)
@@ -2061,7 +2081,7 @@ class NzbObject(TryList):
         if self.bytes_par2 is None:
             self.bytes_par2 = 0
             for nzf in self.files + self.finished_files:
-                if sabnzbd.par2file.is_parfile(nzf.filename):
+                if sabnzbd.par2file.has_par2_in_filename(nzf.filename):
                     self.bytes_par2 += nzf.bytes
         if self.download_path is None:
             self.download_path = long_path(os.path.join(cfg.download_dir.get_path(), self.work_name))
@@ -2179,12 +2199,3 @@ def name_extractor(subject: str) -> str:
 
     # Return the subject
     return subject
-
-
-def matcher(pattern, txt):
-    """Return True if `pattern` is sufficiently equal to `txt`"""
-    if txt.endswith(pattern):
-        txt = txt[: txt.rfind(pattern)].strip()
-        return (not txt) or txt.endswith('"')
-    else:
-        return False
